package de.nwfva.silviculture.base;

/* http://www.nw-fva.de
   Version 05-01-2018

   (c) 2007-18 Juergen Nagel and Jan Hansen, Northwest German Forest Research Station, 
       Grätzelstr.2, 37079 Göttingen, Germany
       E-Mail: Juergen.Nagel@nw-fva.de
       E-Mail: Jan.Hansen@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */
import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.Node;
import treegross.base.SpeciesDefMap;
import treegross.base.Stand;
import treegross.treatment.CropTreeSelection;
import treegross.treatment.CropTreeSpecies;
import treegross.treatment.TreatmentElements2;

/**
 *
 * @author jnagel and jhansen 
 */
public class TeConvertToWet implements TreatmentElement {

    private final String name = "Umbau";
    private final String label = "te_conversion_wet";
    private final String desc = "te_conversion_wet_info";



    public TeConvertToWet() {
        init();
    }


    private void init() {
 
    }

    //logic of treatment element:
    public void makeHarvest(Stand st) {  
        int zeit = 30;
        if (st != null) {
            boolean convert = true;
            boolean startHarvest = false;
            double g1 = 0.0;
            double g2 = 0.0;
            int art1 = Math.floorDiv(st.wet, 10);
            int art2 = st.wet - (art1 * 10);
            for (int i = 0; i < st.nspecies; i++) {
                if (art1 * 100 < st.sp[i].code && st.sp[i].code < (art1 + 1) * 100) {
                    g1 = g1 + st.sp[i].percCSA;
                }
                if (art2 * 100 < st.sp[i].code && st.sp[i].code < (art2 + 1) * 100) {
                    g2 = g2 + st.sp[i].percCSA;
                }
            }
            if (g1 + g2 > 30) {
                convert = false;
            }
            if (art1 == 1 ) zeit = 20;
            if (art1 == 7 ) zeit = 20;
// 
            if (convert || (st.status >30 && st.status < 50)) {
                

                if (st.status < 30) {

                    for (int i = 0; i < st.nspecies; i++) {
                        st.sp[i].trule.numberCropTreesWanted = (int) Math.round(st.sp[i].spDef.cropTreeNumber * st.sp[i].percCSA / 100.0);
                        st.sp[i].trule.targetCrownPercent = st.sp[i].percCSA;
                        st.sp[i].trule.minCropTreeHeight = 18.0;
                        st.sp[i].trule.targetDiameter = st.sp[i].spDef.targetDiameter * 0.8;
                    }
                    CropTreeSpecies ctspecies[] = new CropTreeSpecies[30];
                    /**
                     * Number of crop trees to select
                     */
                    int n_ct_ha;
                    /**
                     * distance of crop trees to select
                     */
                    double dist_ct;
                    //Initialize Croptreespecies dependent on target mixture percentage and height of first thinning
                    for (int i = 0; i < st.nspecies; i++) {
                        if (st.sp[i].trule.numberCropTreesWanted == 0) {
                            dist_ct = 0.01;
                        } //            else dist_ct=0.90* Math.sqrt((st.size*100.0*st.sp[i].trule.targetCrownPercent/(st.sp[i].trule.numberCropTreesWanted*st.size)));
                        else {
                            dist_ct = 0.80 * Math.sqrt((100.0 * st.sp[i].trule.targetCrownPercent / (st.sp[i].trule.numberCropTreesWanted)));
                        }
                        //Initialize CropTreeSpecies
                        ctspecies[i] = new CropTreeSpecies();
                        ctspecies[i].addCtsp(st.sp[i].code, st.sp[i].trule.numberCropTreesWanted, dist_ct, st.sp[i].trule.minCropTreeHeight);
                    }
                    //Select Croptrees dependent on CTSpecies
                    CropTreeSelection ctselect = new CropTreeSelection();
                    ctselect.selectCropTrees(st, ctspecies);
//
//  Pruefen, ob die crop trees erntereif sind 30 % der ct > zielstaerke
//
                    double gct = 0.0;
                    double gzs = 0.0;
                    for (int i = 0; i < st.ntrees; i++) {
                        if (st.tr[i].crop && st.tr[i].out < 0) {
                            gct = gct + Math.PI * Math.pow((st.tr[i].d / 200), 2.0);
                            if (st.tr[i].d > st.tr[i].sp.spDef.targetDiameter * 0.8) {
                                gzs = gzs + Math.PI * Math.pow((st.tr[i].d / 200), 2.0);
                            }
                        }
                    }
                    if (gct > 0.0) {
                        gzs = gzs / gct;
                    }
                    if (gzs > 0.15) {
                        startHarvest = true;
                    }
                }

                st.descspecies();

                if (startHarvest || st.status > 30) {
                    if (st.status < 30) {
                        st.status = 30;
                    }
                    st.status = st.status + 1;
                    TreatmentElements2 te2 = new TreatmentElements2();

                    double degree = st.bha / te2.getMaxStandBasalArea(st, true);
                    double reduction;
                    if (zeit - (5.0 * (st.status - 31.0)) <= 0.0) {
                        reduction = degree;
                    } else {
                        reduction = (5.0 / (zeit - (5.0 * (st.status - 31.0)))) * (st.bha / te2.getMaxStandBasalArea(st, true));
                    }
                    degree = degree - reduction;

                    double baHarv = 0.0;
                    double baOut;
                    baOut = st.bha - te2.getMaxStandBasalArea(st, true) * degree;
                    if (baOut < 0.0) {
                        baOut = 0.0;
                    }
                    while (baHarv < baOut) {
                        // Harvest crop trees
                        double max = -9999999.0;
                        int merk = -9;
                        for (int i = 0; i < st.ntrees; i++) {
                            if (st.tr[i].out < 0 && st.tr[i].d >= 7.0 && !st.tr[i].habitat && st.tr[i].d >= (st.tr[i].sp.trule.targetDiameter * 0.3)) {
                                double diff = st.tr[i].d - st.tr[i].sp.trule.targetDiameter;
                                if (max < diff) {
                                    merk = i;
                                    max = diff;
                                }
                            }
                        }
                        // break if degree to small or no harvest tree found (merk=-9)
                        if ((te2.getDegreeOfCover(0, st, true) < st.trule.minimumCoverage) || merk == -9) {
                            break;
                        }
                        st.tr[merk].out = st.year;
                        st.tr[merk].outtype = 3;
                        //st.tr[merk].no+="_ss";
                        baHarv += Math.PI * Math.pow(st.tr[merk].d / 200.0, 2.0) * (st.tr[merk].fac / st.size);
                    }
                    if (zeit - (5.0 * (st.status - 30.0)) <= 0.1) {
                        st.status = 0;
                        st.bt = st.wet;
                    }

                }


            }
        }
        //System.out.println(name + " executed");
    }

    @Override
    public String toString() {
        return getLabel();
    }

    // interface overrides: 
    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getLabel() {
        return RBHolder.getResourceBundle().getString(label);
    }

    @Override
    public String getDescription() {
        return RBHolder.getResourceBundle().getString(desc);
    }

    @Override
    public void execute(Stand st) {
        makeHarvest(st);
    }

    @Override
    public ArrayList<TreatmentElementParameter> getRequiredParameters() {
        ArrayList<TreatmentElementParameter> params;
        params = new ArrayList<>(0);
        return params;
    }

    @Override
    public void parse(Node n) {
    }

    @Override
    public int getGroup() {
        return TreatmentElement.GROUP_OTHER;
    }

    @Override
    public void setDefaults(SpeciesDefMap sdm, Stand st) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
    
}
