/* http://www.nw-fva.de
   Version 05-01-2018

   (c) 2007-18 Juergen Nagel and Jan Hansen, Northwest German Forest Research Station, 
       Grätzelstr.2, 37079 Göttingen, Germany
       E-Mail: Juergen.Nagel@nw-fva.de
       E-Mail: Jan.Hansen@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */
package de.nwfva.silviculture.base;

import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.Node;
import treegross.base.Competition;
import treegross.base.SpeciesDefMap;
import treegross.base.Stand;
import treegross.treatment.TreatmentElements2;

/**
 *
 * @author jnagel and jhansen
 */
public class TeHarvestByGroupSelection implements TreatmentElement {

    private final String name = "Femelschlag";
    private final String label = "te_harvest_groups";
    private final String desc = "te_harvest_groups_info";

    private double maxVolume;
    private double lochdurchmesser;
    private String criterium;
    private double wert;

    private ArrayList<Object> defCriterium;

    public TeHarvestByGroupSelection(double maxVolume, double lochdurchmesser, String criterium, double wert) {
        init(maxVolume, lochdurchmesser, criterium, wert);
    }

    public TeHarvestByGroupSelection() {
        this(100.0, 15.0, "1 " + RBHolder.getResourceBundle().getString("te_crop30_d"), 60);
    }

    private void init(double maxVolume, double lochdurchmesser, String criterium, double wert) {
        this.maxVolume = maxVolume;
        this.lochdurchmesser = lochdurchmesser;
        this.criterium = criterium;
        this.wert = wert;
        defCriterium = new ArrayList<>(3);
        defCriterium.add("1 " + RBHolder.getResourceBundle().getString("te_crop30_d"));
        defCriterium.add("2 " + RBHolder.getResourceBundle().getString("te_clearcut_age"));
        defCriterium.add("3 " + RBHolder.getResourceBundle().getString("te_clearcut_h100"));
    }

    //logic of treatment element:
    public void makeHarvest(Stand st) {
        //Do nothing but printing line of text to stdout:
        if (st != null) {
            // Create a grid
            double xmin = 999999.0;
            double xmax = 0.0;
            double ymin = 999999.0;
            double ymax = 0.0;
            for (int i = 0; i < st.ncpnt; i++) {
                if (st.cpnt[i].x < xmin) {
                    xmin = st.cpnt[i].x;
                }
                if (st.cpnt[i].y < ymin) {
                    ymin = st.cpnt[i].y;
                }
                if (st.cpnt[i].x > xmax) {
                    xmax = st.cpnt[i].x;
                }
                if (st.cpnt[i].y > ymax) {
                    ymax = st.cpnt[i].y;
                }
            }
            double move = lochdurchmesser / 4.0;

            int nx = (int) Math.round((xmax - xmin) / (lochdurchmesser / 4.0));
            int ny = (int) Math.round((ymax - ymin) / (lochdurchmesser / 4.0));
            double xp, yp;
            Competition comp = new Competition();

            for (int iim = 0; iim < nx; iim++) {
                xp = (iim + 1) * move + xmin;
                for (int jjm = 0; jjm < ny; jjm++) {
                    yp = (jjm + 1) * move + ymin;
// Abprüfen, ob der Punkt auf der Fläche ist sonst weiterspringen
                    if (comp.pnpoly(xp, yp, st) != 0) {

                        boolean startHarvest = false;
                        if (criterium.startsWith("1") && st.status < 50) {
                            // Check if 30% of crop trees > wert
                            double anz = 0;
                            double anzGtWert = 0;
                            for (int i = 0; i < st.ntrees; i++) {
                                double dist = Math.pow((xp - st.tr[i].x), 2.0) + Math.pow((yp - st.tr[i].y), 2.0);
                                if (dist > 0.0 ) dist=Math.sqrt(dist);
                                if (st.tr[i].out < 0 && st.tr[i].d >= 7.0 && dist < lochdurchmesser/2.0 ) {
                                    anz = anz + st.tr[i].v* st.tr[i].fac;
                                    if (st.tr[i].d > wert) {
                                        anzGtWert = anzGtWert + st.tr[i].v * st.tr[i].fac;
                                    }
                                }
                            }

                            if (anz < 1) {
                                st.status = 1;
                            }
                            if (anz > 0.0) {
                                if (anzGtWert /anz > 0.15 ) {
                                    startHarvest = true;
                                }
                            }
                        }
                        if (criterium.startsWith("2") ) {
                            for (int i = 0; i < st.ntrees; i++) {
                                double dist = Math.pow((xp - st.tr[i].x), 2.0) + Math.pow((yp - st.tr[i].y), 2.0);
                                if (dist > 0.0 ) dist=Math.sqrt(dist);
                                if (st.tr[i].out < 0 && st.tr[i].d >= 7.0 && st.tr[i].age >= wert ) {
                                   startHarvest = true;
                                }
                            }
                        }
                        if (criterium.startsWith("3") ) {
                            for (int i = 0; i < st.ntrees; i++) {
                                double dist = Math.pow((xp - st.tr[i].x), 2.0) + Math.pow((yp - st.tr[i].y), 2.0);
                                if (dist > 0.0 ) dist=Math.sqrt(dist);
                                if (st.tr[i].out < 0 && st.tr[i].d >= 7.0 && st.tr[i].h >= wert ) {
                                   startHarvest = true;
                                }
                            }
                        }
                        if (startHarvest ) {
                            TreatmentElements2 te2 = new TreatmentElements2();
                            st.trule.maxHarvestVolume = maxVolume;
                            double harvested = te2.getHarvestedOutVolume(st);
                            double vmaxharvest = st.size * st.trule.maxHarvestVolume - te2.getHarvestedOutVolume(st);

                            if ((harvested < vmaxharvest && vmaxharvest > 0) ) {
                                    //see if there are trees , then harvest those trees
                                    for (int iha = 0; iha < st.ntrees; iha++) {
                                        if (st.tr[iha].habitat == false && st.tr[iha].out < 0 && st.tr[iha].h > 12) {
                                            double dist = Math.pow(xp - st.tr[iha].x, 2.0) + Math.pow(yp - st.tr[iha].y, 2.0);
                                            if (dist != 0.0) {
                                                dist = Math.sqrt(dist);
                                            }
                                            if (dist <= lochdurchmesser/2.0 && st.tr[iha].h > 12.0 ) {
//                                    vout += st.tr[i].fac * st.tr[i].v;
                                                harvested += st.tr[iha].fac * st.tr[iha].v;
                                                st.tr[iha].out = st.year;
                                                st.tr[iha].outtype = 3;
                                            }
                                        }
                                    }
                                }

                        }
                    } // Punkt im Polygon

                }
            }

        }
        System.out.println(name + " executed");
    }

    @Override
    public String toString() {
        return getLabel();
    }

    // interface overrides: 
    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getLabel() {
        return RBHolder.getResourceBundle().getString(label);
    }

    @Override
    public String getDescription() {
        return RBHolder.getResourceBundle().getString(desc);
    }

    @Override
    public void execute(Stand st) {
        makeHarvest(st);
    }

    @Override
    public ArrayList<TreatmentElementParameter> getRequiredParameters() {
        ArrayList<TreatmentElementParameter> params;
        params = new ArrayList<>(3);
        params.add(new TreatmentElementParameter("maxVolume", RBHolder.getResourceBundle().getString("te_maxvolume"), RBHolder.getResourceBundle().getString("te_maxvolume_info"), maxVolume));
        params.add(new TreatmentElementParameter("lochdurchmesser", RBHolder.getResourceBundle().getString("te_diameter"), RBHolder.getResourceBundle().getString("te_diameter_info"), lochdurchmesser));
        params.add(new TreatmentElementParameter("criterium", RBHolder.getResourceBundle().getString("te_criterium"), RBHolder.getResourceBundle().getString("te_condition"), criterium,
                TreatmentElementParameter.CONSTRAINT_TYPE_VALUES, defCriterium));
        params.add(new TreatmentElementParameter("wert", RBHolder.getResourceBundle().getString("te_value"), RBHolder.getResourceBundle().getString("te_greater_than"), wert));
        return params;
    }

    @Override
    public void parse(Node n) {
        List<Node> params = treegross.tools.XmlTool.getChilds(n, TreatmentElementParameter.class.getSimpleName());
        String aName, value;
        for (Node p : params) {
            aName = p.getAttributes().getNamedItem("name").getNodeValue();
            value = p.getAttributes().getNamedItem("value").getNodeValue();
            switch (aName) {
                case "maxVolume":
                    maxVolume = Double.parseDouble(value);
                    break;
                case "lochdurchmesser":
                    lochdurchmesser = Double.parseDouble(value);
                    break;
                case "criterium":
                    criterium = value;
                    break;
                case "wert":
                    wert = Double.parseDouble(value);
                    break;
            }
        }
    }

    @Override
    public int getGroup() {
        return TreatmentElement.GROUP_HARVESTING;
    }

    @Override
    public void setDefaults(SpeciesDefMap sdm, Stand st) {
        //init with class values 
        init(100.0, 15.0, "1 "+RBHolder.getResourceBundle().getString("te_clearcut_d100"), 60);
        // try to init with stand values if stand is not null
        if (st != null) {
            if (st.sp.length > 0) {
                wert = st.sp[0].trule.targetDiameter;
                return;
            }
        }
        //if st is null use beach values from species def map        
        if (sdm != null && sdm.getByCode(211) != null) {
            wert = sdm.getByCode(211).targetDiameter;
        }
    }
   
}
