/* http://www.nw-fva.de
   Version 05-01-2018

   (c) 2007-18 Juergen Nagel and Jan Hansen, Northwest German Forest Research Station, 
       Grätzelstr.2, 37079 Göttingen, Germany
       E-Mail: Juergen.Nagel@nw-fva.de
       E-Mail: Jan.Hansen@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */
package de.nwfva.silviculture.base;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.w3c.dom.Node;
import treegross.base.Competition;
import treegross.base.SpeciesDefMap;
import treegross.base.SpeciesNotDefinedException;
import treegross.base.Stand;

/**
 *
 * @author jnagel and jhansen
 */
public class TePlantUnCanopy implements TreatmentElement {

    private final String name = "PlantUC";
    private final String label = "te_plantUC";
    private final String desc = "te_plantUC_info";

    private double schlussgrad;
    private String material;
    private double verbandx;
    private double verbandy;
    private int onetime = 0;

    public TePlantUnCanopy(double schlussgrad, String material, double verbandx, double verbandy) {
        init(schlussgrad, material, verbandx, verbandy);
    }

    public TePlantUnCanopy() {
        this(0.3, "211=0.8=32.4;", 2.0, 3.0);
    }

    private void init(double schlussgrad, String material, double verbandx, double verbandy) {
        this.schlussgrad = schlussgrad;
        this.material = material;
        this.verbandx = verbandx;
        this.verbandy = verbandy;
    }

    //logic of treatment element:
    public void makePlanting(Stand st) {
        //Do nothing but printing line of text to stdout:

        
// Pflanzstring zerlegen
        String[] sMaterial = material.split(";");
        int[] nArt = new int[sMaterial.length];
        double[] nArtPro = new double[sMaterial.length];
        double[] nArtBon = new double[sMaterial.length];
        int nArten = 0;
        try {
            for (int i = 0; i < sMaterial.length; i++) {
                String[] txt = sMaterial[i].split("=");
                nArt[i] = Integer.parseInt(txt[0]);
                nArtPro[i] = Double.parseDouble(txt[1]);
                nArtBon[i] = Double.parseDouble(txt[2]);
                nArten = nArten + 1;
            }
        } catch (Exception e) {
            System.out.println(e.toString());
        }

//  Deckungsgrad der zupflanzenden Arten berechnen
        double dgrad = 0.0;
        for (int i = 0; i < st.ntrees; i++) {
            for (int k = 0; k < nArten; k++) {
                if (st.tr[i].out < 0 && nArt[k] == st.tr[i].code) {
                    dgrad = dgrad + Math.PI * st.tr[i].cw / 2 * st.tr[i].fac;
                }
            }
        }
        dgrad = dgrad / (st.size*10000);
// Nur Unterpflanzen, wenn mindestens ein Schlussgrad von 0.4 vorhanden fuer Pflanzen mit einer Hoehe
// > 28.0 m ist, weil
// sonst in Uberhaltsituation auch unterpflanzt wird.
        double sgrad = 0.0;
        for (int i = 0; i < st.ntrees; i++) {
            if (st.tr[i].out < 0 && st.tr[i].h > 18.0) {
                sgrad += Math.PI * st.tr[i].cw / 2 * st.tr[i].fac;
            }
        }
        sgrad = sgrad / (st.size*10000);
//      
        if (dgrad <= schlussgrad && sgrad > 0.4) {
            Competition comp = new Competition();
            if (st != null) {
// Den Deckungsgrad kulmulativ umarbeiten
                for (int kk = 1; kk < nArten; kk++) {
                    nArtPro[kk] = nArtPro[kk - 1] + nArtPro[kk];
                }


// Create a grid
                double xmin = 999999.0;
                double xmax = 0.0;
                double ymin = 999999.0;
                double ymax = 0.0;
                for (int i = 0; i < st.ncpnt; i++) {
                    if (st.cpnt[i].x < xmin) {
                        xmin = st.cpnt[i].x;
                    }
                    if (st.cpnt[i].y < ymin) {
                        ymin = st.cpnt[i].y;
                    }
                    if (st.cpnt[i].x > xmax) {
                        xmax = st.cpnt[i].x;
                    }
                    if (st.cpnt[i].y > ymax) {
                        ymax = st.cpnt[i].y;
                    }
                }
                int nx = (int) Math.round((xmax - xmin) / verbandx);
                int ny = (int) Math.round((ymax - ymin) / verbandy);
                double cover, xp, yp;

// make Grid 1 by 1 m and find position with less crown coverage                  
                for (int i = 0; i < nx; i++) {
                    xp = (i + 1) * verbandx + xmin;
                    for (int j = 0; j < ny; j++) {
                        yp = (j + 1) * verbandy + ymin;
// Prüfen, ob der Pfanzplatz schon besetzt ist im Radius 1 m                        
                        boolean frei = true;
                        double ent =0.0;
                          for (int jj = 0; jj < st.ntrees; jj++) {
                              if (st.tr[jj].out < 0){
                                  ent = Math.pow(st.tr[jj].x-xp, 2.0)+ Math.pow(st.tr[jj].y-yp, 2.0);
                                  if (ent > 0.0) ent = Math.sqrt(ent);
                                  if (ent < 1.0) {
                                      frei = false;
                                      break;
                                  }
                              }
                          }
// Abprüfen, ob der Punkt auf der Fläche ist sonst weiterspringen
                        if (comp.pnpoly(xp, yp, st) != 0 && frei) {
// Berechnen, wieviel Überdeckung und wieviel in der Fläche mittels 1x1 Raster

                            double ran = st.random.nextUniform();
                            int choArt = 0;
                            double siteindex = 30.0;
                            if (ran < nArtPro[0]) {
                                choArt = nArt[0];
                                siteindex = nArtBon[0];
                            }
                            for (int kk = 1; kk < nArten; kk++) {
                                if (ran >= nArtPro[kk - 1] && ran < nArtPro[kk]) {
                                    choArt = nArt[kk];
                                    siteindex = nArtBon[kk];
                                }
                            }

                            try {
                                if (choArt > 0) {
                                    double h = 0.8 + st.random.nextUniform() * 0.2;
  
                                    st.addTreeFromPlanting(choArt, "pU" + st.ntrees + "_" + st.year, 1, -1, 1.0, h, 0.1, 2.52, siteindex, xp, yp, 0, 0, 0, 0);
                                }
                            } catch (SpeciesNotDefinedException ex) {
                                Logger.getLogger(this.getClass().getName()).log(Level.WARNING, "", ex);
                            }

                        } // Punkt im Polygon

                    }
                }
            }

        }
        System.out.println(name + " UC executed");
    }

    @Override
    public String toString() {
        return getLabel();
    }

    // interface overrides: 
    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getLabel() {
        return RBHolder.getResourceBundle().getString(label);
    }

    @Override
    public String getDescription() {
        return RBHolder.getResourceBundle().getString(desc);
    }

    @Override
    public void execute(Stand st) {
        makePlanting(st);
    }

    @Override
    public ArrayList<TreatmentElementParameter> getRequiredParameters() {
        ArrayList<TreatmentElementParameter> params;
        params = new ArrayList<>(3);
        params.add(new TreatmentElementParameter("schlussgrad", RBHolder.getResourceBundle().getString("te_mcoverage"), RBHolder.getResourceBundle().getString("te_mcoverage_info"), schlussgrad));
        params.add(new TreatmentElementParameter("material", RBHolder.getResourceBundle().getString("te_plants"), RBHolder.getResourceBundle().getString("te_plants_info"), material));
        params.add(new TreatmentElementParameter("verbandx", RBHolder.getResourceBundle().getString("te_plant_spacing"), RBHolder.getResourceBundle().getString("te_plants_spacing_info1"), verbandx));
        params.add(new TreatmentElementParameter("verbandy", RBHolder.getResourceBundle().getString("te_plant_spacing"), RBHolder.getResourceBundle().getString("te_plants_spacing_info2"), verbandy));
        return params;
    }

    @Override
    public void parse(Node n) {
        List<Node> params = treegross.tools.XmlTool.getChilds(n, TreatmentElementParameter.class.getSimpleName());
        String aName, value;
        for (Node p : params) {
            aName = p.getAttributes().getNamedItem("name").getNodeValue();
            value = p.getAttributes().getNamedItem("value").getNodeValue();
            switch (aName) {
                case "material":
                    material = value;
                    break;
                case "schlussgrad":
                    schlussgrad = Double.parseDouble(value);
                    break;
                case "verbandx":
                    verbandx = Double.parseDouble(value);
                    break;
                case "verbandy":
                    verbandy = Double.parseDouble(value);
                    break;
            }
        }
    }

    @Override
    public int getGroup() {
        return TreatmentElement.GROUP_OTHER;
    }

    @Override
    public void setDefaults(SpeciesDefMap sdm, Stand st) {
        init(0.3, "211=0.3=32.4;", 2.0, 3.0);
        onetime =0;
    }
    
}
