/* http://www.nw-fva.de
   Version 05-01-2018

   (c) 2007-18 Juergen Nagel and Jan Hansen, Northwest German Forest Research Station, 
       Grätzelstr.2, 37079 Göttingen, Germany
       E-Mail: Juergen.Nagel@nw-fva.de
       E-Mail: Jan.Hansen@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */
package de.nwfva.silviculture.base;

import java.util.ArrayList;
import org.w3c.dom.Node;
import treegross.base.SpeciesDefMap;
import treegross.base.Stand;

/**
 *
 * @author jnagel and jhansen
 */
public class TeResetHabitatTrees implements TreatmentElement {

    private final String name = "reset habitat trees";
    private final String label = "te_reset";
    private final String desc = "te_reset_habitat";

    public TeResetHabitatTrees() {
        init();
    }

    private void init() {

    }

    //logic of treatment element:
    public void makeSelection(Stand st) {
        //Do nothing but printing line of text to stdout:
        if (st != null) {
            for (int i = 0; i < st.ntrees; i++) {
                if (st.tr[i].habitat && st.tr[i].out < 0 && st.tr[i].d >= 7.0) {
                    st.tr[i].habitat = false;
                }
            }
        }
        //System.out.println(name + " executed. Habitatbäume: "  );
    }

    @Override
    public String toString() {
        return getLabel();
    }

    // interface overrides: 
    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getLabel() {
        return RBHolder.getResourceBundle().getString(label);
    }

    @Override
    public String getDescription() {
        return RBHolder.getResourceBundle().getString(desc);
    }

    @Override
    public void execute(Stand st) {
        makeSelection(st);
    }

    @Override
    public ArrayList<TreatmentElementParameter> getRequiredParameters() {
        ArrayList<TreatmentElementParameter> params;
        params = new ArrayList<>(3);
        return params;
    }

    @Override
    public void parse(Node n) {
        //ToDo Error handling
        /*
         List<Node> params = treegross.tools.XmlTool.getChilds(n, TreatmentElementParameter.class.getSimpleName());
         String aName, value;
         for (Node p : params) {            
         aName = p.getAttributes().getNamedItem("name").getNodeValue();
         value = p.getAttributes().getNamedItem("value").getNodeValue();            
         //System.out.println("parsing: "+aName+" value: " +value);

         }
         */
    }

    @Override
    public int getGroup() {
        return TreatmentElement.GROUP_NAT_CONVERSATION;
    }

    @Override
    public void setDefaults(SpeciesDefMap sdm, Stand st) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
    
}
