/* http://www.nw-fva.de
   Version 05-01-2018

   (c) 2007-18 Juergen Nagel and Jan Hansen, Northwest German Forest Research Station, 
       Grätzelstr.2, 37079 Göttingen, Germany
       E-Mail: Juergen.Nagel@nw-fva.de
       E-Mail: Jan.Hansen@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */
package de.nwfva.silviculture.base;

import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.Node;
import treegross.base.FunctionInterpreter;
import treegross.base.SpeciesDefMap;
import treegross.base.Stand;

/**
 *
 * @author jnagel and jhansen
 */
public class TeSelectCropTrees implements TreatmentElement {

    private final String name = "crop tree selection";
    private final String label = "te_croptree";
    private final String desc = "te_croptree_info";

    private int[] code = new int[3];
    private String[] nha = new String[50];
    private double[] mixpercent = new double[3];;
    private boolean useQuality;

    private ArrayList<Object> defhardwoodOnly;

    public TeSelectCropTrees(int code1, String nha1, double mixpercent1, int code2, String nha2, double mixpercent2,
            int code3, String nha3, double mixpercent3, boolean useQuality) {
        init(code1, nha1, mixpercent1, code2, nha2, mixpercent2, code3, nha3, mixpercent3, useQuality);
    }

    public TeSelectCropTrees() {
        this(0, "0", 100.0, 0, "0", 0.0, 0, "0", 0.0, false);
    }

    private void init(int code1, String nha1, double mixpercent1, int code2, String nha2, double mixpercent2,
            int code3, String nha3, double mixpercent3,boolean useQuality) {
        this.code[0] = code1;
        this.nha[0] = nha1;
        this.mixpercent[0] = mixpercent1;
        this.code[1] = code2;
        this.nha[1] = nha2;
        this.mixpercent[1] = mixpercent2;
        this.code[2] = code3;
        this.nha[2] = nha3;
        this.mixpercent[2] = mixpercent3;
        this.useQuality = useQuality;

    }

    //logic of treatment element:
    public void makeSelection(Stand st) {
        //Do nothing but printing line of text to stdout:
        if (st != null) {
            double hmin = 9000.0;
            for (int i = 0; i < st.nspecies; i++) {
                if (st.sp[i].hg < hmin && st.sp[i].hg > 1.3) {
                    hmin = st.sp[i].hg - 1.0;
                }
            }
// Baumarten min max setzen, z.B. damit bei 110 alle Eichen geladen werden 
            int[] codemin = new int[3];
            int[] codemax = new int[3];
            for (int i = 0; i < 3; i++) {
                codemin[i] = code[i];
                codemax[i] = code[i];
                if (code[i] >= 100) {
                    String ctxt = Integer.toString(code[i]);
                    if (ctxt.endsWith("0")) {
                        codemin[i] = Integer.parseInt(ctxt.substring(0, 2) + "0");
                        codemax[i] = Integer.parseInt(ctxt.substring(0, 2) + "9");
                    }
                }
            }
            

// new crop tree settings laden
            for (int i = 0; i < st.nspecies; i++) {
                st.sp[i].trule.numberCropTreesWanted = 0;
                st.sp[i].trule.targetCrownPercent = 0.0;
                st.sp[i].trule.minCropTreeHeight = 10.0;
                double d100 = st.sp[i].d100;
                if (d100 < 7) d100 = st.sp[i].dg;
                d100 = 10000.0/ (Math.PI*Math.pow(d100/200.0,2.0));
                    FunctionInterpreter fi = new FunctionInterpreter();
               
                if (codemin[0] <= st.sp[i].code && st.sp[i].code <= codemax[0]) {
                    
                    int nhax = (int) fi.getValueForTree(st.tr[0], nha[0]);
                    if (nhax < d100) d100= nhax; 
                    st.sp[i].trule.numberCropTreesWanted = (int) Math.round(d100 * mixpercent[0] / 100.0);
                    st.sp[i].trule.targetCrownPercent = mixpercent[0];
                    st.sp[i].trule.minCropTreeHeight = hmin;
                }
                if (codemin[1] <= st.sp[i].code && st.sp[i].code <= codemax[1]) {
                    int nhax = (int) fi.getValueForTree(st.tr[0], nha[1]);
                    if (nhax < d100) d100= nhax; 
                    st.sp[i].trule.numberCropTreesWanted = (int) Math.round(d100 * mixpercent[1] / 100.0);
                    st.sp[i].trule.targetCrownPercent = mixpercent[1];
                    st.sp[i].trule.minCropTreeHeight = hmin;
                }
                if ((codemin[2] <= st.sp[i].code && st.sp[i].code <= codemax[2])) {
                    int nhax = (int) fi.getValueForTree(st.tr[0], nha[2]);
                    if (nhax < d100) d100= nhax; 
                    st.sp[i].trule.numberCropTreesWanted = (int) Math.round(d100 * mixpercent[2] / 100.0);
                    st.sp[i].trule.targetCrownPercent = mixpercent[2];
                    st.sp[i].trule.minCropTreeHeight = hmin;
                }
            }
            CropTreeSpecies2 ctspecies[] = new CropTreeSpecies2[30];
            /**
             * Number of crop trees to select
             */
            int n_ct_ha;
            /**
             * distance of crop trees to select
             */
            double dist_ct;
            //Initialize Croptreespecies dependent on target mixture percentage and height of first thinning
            for (int i = 0; i < st.nspecies; i++) {
                if (st.sp[i].trule.numberCropTreesWanted == 0) {
                    dist_ct = 0.01;
                } //            else dist_ct=0.90* Math.sqrt((st.size*100.0*st.sp[i].trule.targetCrownPercent/(st.sp[i].trule.numberCropTreesWanted*st.size)));
                else {
                    dist_ct = 0.80 * Math.sqrt((100.0 * st.sp[i].trule.targetCrownPercent / (st.sp[i].trule.numberCropTreesWanted)));
                }
                //Initialize CropTreeSpecies
                ctspecies[i] = new CropTreeSpecies2();
                ctspecies[i].addCtsp(st.sp[i].code, st.sp[i].trule.numberCropTreesWanted, dist_ct, st.sp[i].trule.minCropTreeHeight);
            }
            //Select Croptrees dependent on CTSpecies
            CropTreeSelection2 ctselect = new CropTreeSelection2();
            ctselect.selectCropTrees(st, ctspecies, useQuality);
        }
        System.out.println(name + "crop trees executed. ");
    }

    @Override
    public String toString() {
        return getLabel();
    }

    // interface overrides: 
    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getLabel() {
        return RBHolder.getResourceBundle().getString(label);
    }

    @Override
    public String getDescription() {
        return RBHolder.getResourceBundle().getString(desc);
    }

    @Override
    public void execute(Stand st) {
        makeSelection(st);
    }

    @Override
    public ArrayList<TreatmentElementParameter> getRequiredParameters() {
        ArrayList<TreatmentElementParameter> params;
        params = new ArrayList<>(3);
        params.add(new TreatmentElementParameter("code1", "1. "+RBHolder.getResourceBundle().getString("te_cropspecies"), RBHolder.getResourceBundle().getString("te_cropspecies_info"), code[0]));
        params.add(new TreatmentElementParameter("nha1", RBHolder.getResourceBundle().getString("te_ncrop"), RBHolder.getResourceBundle().getString("te_ncrop_info"), nha[0]));
        params.add(new TreatmentElementParameter("mixpercent1", RBHolder.getResourceBundle().getString("te_mix"), RBHolder.getResourceBundle().getString("te_mix_info"), mixpercent[0]));
        params.add(new TreatmentElementParameter("code2", "2. "+RBHolder.getResourceBundle().getString("te_cropspecies"), RBHolder.getResourceBundle().getString("te_cropspecies_info"), code[1]));
        params.add(new TreatmentElementParameter("nha2", RBHolder.getResourceBundle().getString("te_ncrop"), RBHolder.getResourceBundle().getString("te_ncrop_info"), nha[1]));
        params.add(new TreatmentElementParameter("mixpercent2",  RBHolder.getResourceBundle().getString("te_mix"), RBHolder.getResourceBundle().getString("te_mix_info"), mixpercent[1]));
        params.add(new TreatmentElementParameter("code3", "3. "+RBHolder.getResourceBundle().getString("te_cropspecies"), RBHolder.getResourceBundle().getString("te_cropspecies_info"), code[2]));
        params.add(new TreatmentElementParameter("nha3", RBHolder.getResourceBundle().getString("te_ncrop"), RBHolder.getResourceBundle().getString("te_ncrop_info"), nha[2]));
        params.add(new TreatmentElementParameter("mixpercent3",  RBHolder.getResourceBundle().getString("te_mix"), RBHolder.getResourceBundle().getString("te_mix_info"), mixpercent[2]));
        params.add(new TreatmentElementParameter("useQuality", RBHolder.getResourceBundle().getString("te_useQuality"), RBHolder.getResourceBundle().getString("te_useQuality_info"), useQuality));

        return params;
    }

    @Override
    public void parse(Node n) {
        //ToDo Error handling
        List<Node> params = treegross.tools.XmlTool.getChilds(n, TreatmentElementParameter.class.getSimpleName());
        String aName, value;
        for (Node p : params) {
            aName = p.getAttributes().getNamedItem("name").getNodeValue();
            value = p.getAttributes().getNamedItem("value").getNodeValue();
            //System.out.println("parsing: "+aName+" value: " +value);
            switch (aName) {
                case "code1":
                    code[0] = Integer.parseInt(value);
                    break;
                case "nha1":
                    nha[0] = value;
                    break;
                case "mixpercent1":
                    mixpercent[0] = Double.parseDouble(value);
                    break;
                case "code2":
                    code[1] = Integer.parseInt(value);
                    break;
                case "nha2":
                    nha[1] = value;
                    break;
                case "mixpercent2":
                    mixpercent[1] = Double.parseDouble(value);
                    break;
                case "code3":
                    code[2] = Integer.parseInt(value);
                    break;
                case "nha3":
                    nha[2] = value;
                    break;
                case "mixpercent3":
                    mixpercent[2] = Double.parseDouble(value);
                    break;
                case "useQuality":
                    useQuality = Boolean.parseBoolean(value);
                    break;
            }
        }
    }

    @Override
    public int getGroup() {
        return TreatmentElement.GROUP_PREPARATION;
    }

    @Override
    public void setDefaults(SpeciesDefMap sdm, Stand st) {
        int[] index = { -9,-9,-9 };
        for (int i=0; i < 3; i++){
            double gmax = -9.0;
            for (int j=0; j < st.nspecies; j++){
                if (gmax < st.sp[j].gha && index[0] != j && index[1] != j ){
                    gmax = st.sp[j].gha;
                    index[i] = j;
                }
            }
         }
        //
        for (int i=0; i < 3; i++){
            if (index[i] >= 0){
                code[i] = st.sp[index[i]].code;
                nha[i] = Integer.toString(st.sp[index[i]].spDef.cropTreeNumber);
                mixpercent[i] = st.sp[index[i]].percCSA;
            }
        }
        // Abgleich des Mischungsprozents
        double sum =0 ;
        for (int i=0; i < 3; i++){
            sum = sum + mixpercent[i];
        }
        for (int i=0; i < 3; i++){
            mixpercent[i] = Math.round(10*mixpercent[i] *100.0 /sum)/10;
        }
        init(code[0], nha[0], mixpercent[0] , code[1], nha[1], mixpercent[1], code[2], nha[2], mixpercent[2],false);
    }
    
}
