/* http://www.nw-fva.de
   Version 05-01-2018

   (c) 2007-18 Jan Hansen, Northwest German Forest Research Station, 
       Grätzelstr.2, 37079 Göttingen, Germany
       E-Mail: Jan.Hansen@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */
package de.nwfva.silviculture.base;

import java.util.ArrayList;
import java.util.Locale;
import org.w3c.dom.Node;
import treegross.base.SpeciesDefMap;
import treegross.base.Stand;

/**
 *
 * @author jhansen
 *
 * attention: all implementations must have an empty constructor!! and for gui
 * models (lists etc) all implementations should overwide toString method
 *
 */
public interface TreatmentElement {

    public static final int GROUP_NAT_CONVERSATION = 0;
    public static final int GROUP_PREPARATION = 1;
    public static final int GROUP_THINNING = 2;
    public static final int GROUP_HARVESTING = 3;
    public static final int GROUP_OTHER = 100;
   
    public String getName();

    public String getLabel();

    public String getDescription();

    public int getGroup();

    public void execute(treegross.base.Stand stand);

    public ArrayList<TreatmentElementParameter> getRequiredParameters();

    public void parse(Node n) throws NumberFormatException;
    
    /**
     * Set default values from <code>treegross.base.SpeciesDefMap</code>.
     * If param st (<code>treegross.baseStand</code>) is not null the given stand should be considered.
     * E.g the target diameter of the leading species. 
     * 
     * @param sdm the <code>treegross.base.SpeciesDefMap</code> to get values from
     * @param st the (optional) Stand to 
     */
    public void setDefaults(SpeciesDefMap sdm, Stand st);

}
