/* http://www.nw-fva.de
   Version 2013-01-13

   (c) 2002 Juergen Nagel, Northwest German Forest Research Station, 
       Grtzelstr.2, 37079 Gttingen, Germany
       E-Mail: Juergen.Nagel@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */

package nwfva.biomass;

import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import nwfva.assortment.StartBrowser;
import treegross.base.*;
import treegross.tools.XmlToHtml;

/**
 * Biomassedialog fr den ForestSimulator
 * @author J. Nagel
 */
public class BiomassDialog extends javax.swing.JDialog {
    private static final Logger log = Logger.getLogger( nwfva.assortment.NWFVA_Nutzung.class.getName() );
    String progDir = "";
    String workDir = "";
    Stand st = null;
    String biomassXML ="";
    String assortmentXML ="";
/**
 * 
 * @param parent
 * @param modal
 * @param stand TreeGrOSS Stand
 * @param programDir Programmverzeichnis
 * @param interactive Mglichkeit andere xml Files zu lesen
 * @param workingDir Ausgabeverzeichnis
 */
    public BiomassDialog(java.awt.Frame parent, boolean modal, Stand stand, String programDir, boolean interactive,
        String workingDir, FileHandler logHandler) {
        super(parent, modal);
        initComponents();
        this.workDir = workingDir;
        if (logHandler == null){
          try{
           Handler handler = new FileHandler("log.txt",true );
           handler.setFormatter(new SimpleFormatter());
           log.addHandler(handler );
           log.info( "Biomass-Dialog gestartet" );
          }
          catch (Exception e){};
        }
        else log.addHandler(logHandler );
        log.info("Biomass- Dialog");
        st = stand;
        String Test = programDir;
        this.progDir = programDir;
        biomassXML = programDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")
                +"biomass"+System.getProperty("file.separator")+"BiomassEnna.xml";
        jTextField1.setText(biomassXML);
        assortmentXML = workDir+System.getProperty("file.separator")+"assortmentlist.xml";
        jTextField2.setText(assortmentXML);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jTextField1 = new javax.swing.JTextField();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        jPanel3 = new javax.swing.JPanel();
        jTextField2 = new javax.swing.JTextField();
        jButton4 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton6 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Biomassenberechnung");

        jPanel1.setLayout(new java.awt.GridLayout(7, 0));

        jLabel4.setFont(new java.awt.Font("Ubuntu", 1, 15)); // NOI18N
        jLabel4.setText("Die Berechnung der Biomassen und Elementgehalt erfolgt in 3 Schritten:");
        jPanel1.add(jLabel4);

        jLabel1.setText("1: Whlen Sie die Datei mit den Biomassefunktionen und Konzentrationswerten ");
        jPanel1.add(jLabel1);

        jPanel2.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jTextField1.setText("jTextField1");
        jPanel2.add(jTextField1);

        jButton2.setText("auswhlen");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel2.add(jButton2);

        jButton3.setText("anzeigen");
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jPanel2.add(jButton3);

        jPanel1.add(jPanel2);

        jLabel2.setText("2. Whlen Sie die XML Datei mit den Sortimenten aus dem Sortimentierungsdialog");
        jPanel1.add(jLabel2);

        jPanel3.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jTextField2.setText("jTextField2");
        jPanel3.add(jTextField2);

        jButton4.setText("auswhlen");
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });
        jPanel3.add(jButton4);

        jButton5.setText("anzeigen");
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });
        jPanel3.add(jButton5);

        jPanel1.add(jPanel3);

        jLabel3.setText("3. Berechnen Sie die Biomasse und Nhrstoffe fr die Sortimente");
        jPanel1.add(jLabel3);

        jButton1.setText("Entnahmemengen auf der Basis der Sortimente berechnen");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel4.add(jButton1);

        jButton6.setText("?");
        jButton6.setToolTipText("Help");
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });
        jPanel4.add(jButton6);

        jPanel1.add(jPanel4);

        getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        setSize(new java.awt.Dimension(1005, 646));
        setLocationRelativeTo(null);
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed

// Ausgabe
        JFileChooser fc2 = new JFileChooser();
        FileFilter filter2 = new FileNameExtensionFilter("XML-Datei","xml"); 
        fc2.setCurrentDirectory(new java.io.File(workDir));
        fc2.setDialogTitle("XML Datei zum Speichern auswhlen");
        fc2.addChoosableFileFilter(filter2);
        fc2.setFileFilter(filter2);
        int auswahl2 = fc2.showOpenDialog(this);
        String xmlname2 = fc2.getSelectedFile().getPath();
        Biomass biomass = new Biomass();
        biomass.loadnbs(biomassXML);
        biomass.calcAssortmentlist(assortmentXML, xmlname2);
        
// Anzeigen
        XmlToHtml xmltohtml = new XmlToHtml();
        String xslfile = progDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")
                +"biomass"+System.getProperty("file.separator")+"nutrient.xsl";
        String htmlfile = xmlname2.replace(".xml", ".html");
        xmltohtml.convert(xmlname2, xslfile, htmlfile);
        StartBrowser startBrowser = new StartBrowser(htmlfile);
        startBrowser.start();
        
        // TODO add your handling code here:
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        // TODO add your handling code here:
        String seite = biomassXML;
        XmlToHtml xmltohtml = new XmlToHtml();
        String xslfile = seite.replace(".xml", ".xsl");
        String htmlfile = workDir+System.getProperty("file.separator")+"Ausgabe.html";
        xmltohtml.convert(seite, xslfile, htmlfile);
        StartBrowser startBrowser = new StartBrowser(htmlfile);
        startBrowser.start();


    }//GEN-LAST:event_jButton3ActionPerformed

    private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
        // TODO add your handling code here:
        String seite = assortmentXML;
        XmlToHtml xmltohtml = new XmlToHtml();
        String htmlfile = seite.replace(".xml", ".html");
        String xslfile  = progDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")+
                "assortment"+System.getProperty("file.separator")+"assortmentlist.xsl";
        xmltohtml.convert(seite, xslfile, htmlfile);
        StartBrowser startBrowser = new StartBrowser(htmlfile);
        startBrowser.start();
    
    }//GEN-LAST:event_jButton5ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        // TODO add your handling code here:
        JFileChooser fc = new JFileChooser();
        FileFilter filter = new FileNameExtensionFilter("XML-Datei", "xml");
        fc.setCurrentDirectory(new java.io.File(workDir));
        fc.setDialogTitle("XML Datei mit der Sortimentierung auswhlen");
        fc.addChoosableFileFilter(filter);
        fc.setFileFilter(filter);
        int auswahl = fc.showOpenDialog(this);
        assortmentXML = fc.getSelectedFile().getPath();
        jTextField2.setText(assortmentXML);
    }//GEN-LAST:event_jButton4ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        // TODO add your handling code here:
        JFileChooser fc = new JFileChooser();
        FileFilter filter = new FileNameExtensionFilter("XML-Datei", "xml");
        fc.setCurrentDirectory(new java.io.File(progDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")
                +"biomass"));
        fc.setDialogTitle("XML Datei mit Biomasse- + Konzentrationswerten auswhlen");
        fc.addChoosableFileFilter(filter);
        fc.setFileFilter(filter);
        int auswahl = fc.showOpenDialog(this);
        biomassXML = fc.getSelectedFile().getPath();
        jTextField1.setText(biomassXML);
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
        // TODO add your handling code here:
        String fileName = st.programDir + System.getProperty("file.separator") + "help" + System.getProperty("file.separator") + "FSUtilization.pdf";
        String seite = "file:" + System.getProperty("file.separator") + System.getProperty("file.separator") + System.getProperty("file.separator") + fileName;
        StartBrowser startBrowser = new StartBrowser(seite);
        startBrowser.start();
    }//GEN-LAST:event_jButton6ActionPerformed
    


    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton6;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField2;
    // End of variables declaration//GEN-END:variables
    
}
