/* http://www.nw-fva.de
   Version 2013-01-13

   (c) 2002 Juergen Nagel, Northwest German Forest Research Station, 
       Grtzelstr.2, 37079 Gttingen, Germany
       E-Mail: Juergen.Nagel@nw-fva.de
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT  WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */

package nwfva.carboncalc;

import java.io.File;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import nwfva.assortment.LoggingPanel;
import nwfva.assortment.LoggingSortiment;
import nwfva.assortment.StartBrowser;
import treegross.base.*;
import treegross.tools.XmlToHtml;

/**
 * Biomassedialog fr den ForestSimulator
 * @author J. Nagel
 */
public class CarbonDialog extends javax.swing.JDialog {
    private static final Logger log = Logger.getLogger( nwfva.assortment.NWFVA_Nutzung.class.getName() );
    String progDir = "";
    String workDir = "";
    String dataDir = "";
    Stand st = null;
    LoggingPanel lp = null;
    Locale currentLocale;
    ResourceBundle messages;
    String carbonResultsXML ="";
    String carbonAssortmentsXML ="";
    String biomassXML = "";
    String cascadeXML = "";
    String carbonresultXML = "";
    /**
 * 
 * @param parent
 * @param modal
 * @param stand TreeGrOSS Stand
 * @param programDir Programmverzeichnis
 * @param interactive Mglichkeit andere xml Files zu lesen
 * @param workingDir Ausgabeverzeichnis
 */
    public CarbonDialog(java.awt.Frame parent, boolean modal, Stand stand, String programDir, boolean interactive,
        String workingDir , String dataDir , String preferredLanguage, FileHandler logHandler) {
        super(parent, modal);
        initComponents();        
        this.workDir = workingDir;
        this.dataDir = dataDir;
        this.progDir = programDir;
        if (logHandler == null){
          try{
           Handler handler = new FileHandler("log.txt",true );
           handler.setFormatter(new SimpleFormatter());
           log.addHandler(handler );
           log.info( "Carbon-Dialog gestartet" );
          }
          catch (Exception e){};
        }
        else log.addHandler(logHandler );
        log.info("Carbon- Dialog");
        st = stand;
        String Test = programDir;
        this.progDir = programDir;
        carbonAssortmentsXML = programDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")
                +"carbon"+System.getProperty("file.separator")+"carbonassortments.xml";
        biomassXML = programDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")
                +"biomass"+System.getProperty("file.separator")+"BiomassEnna.xml";
        cascadeXML = programDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")
                +"carbon"+System.getProperty("file.separator")+"carboncascade.xml";
        carbonresultXML = workDir+System.getProperty("file.separator")+"carbonresult.xml";   
        lp = new LoggingPanel(st, programDir, interactive, workingDir,preferredLanguage, carbonAssortmentsXML);
//        System.out.println(programDir+"  WD:"+workingDir);
        jPanel3.add(lp);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel5 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Kohlenstoffberechnung");

        jPanel5.setLayout(new java.awt.BorderLayout());

        jPanel3.setLayout(new java.awt.BorderLayout());
        jPanel5.add(jPanel3, java.awt.BorderLayout.CENTER);

        jButton1.setText("XML-Treegross Dateien auswhlen und berechnen");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton1);

        jButton2.setText("?");
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel1.add(jButton2);

        jPanel5.add(jPanel1, java.awt.BorderLayout.SOUTH);

        getContentPane().add(jPanel5, java.awt.BorderLayout.CENTER);

        setSize(new java.awt.Dimension(1005, 646));
        setLocationRelativeTo(null);
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        // TODO add your handling code here:

        LoggingSortiment ls[] = null;
// ausgewhlte Sortimente holen        
        ls = lp.getLoogingSortiments();
// Datenfiles auswhlen
        JFileChooser fc2 = new JFileChooser();
        FileFilter filter2 = new FileNameExtensionFilter("XML-Datei","xml"); 
        fc2.setCurrentDirectory(new java.io.File(dataDir));
        fc2.setDialogTitle("XML Datei(en) auswhlen");
        fc2.addChoosableFileFilter(filter2);
        fc2.setFileFilter(filter2);
        fc2.setMultiSelectionEnabled(true);
        int auswahl2 = fc2.showOpenDialog(this);
        File[] selectedFiles = fc2.getSelectedFiles();
//
/*        fc2.setCurrentDirectory(new java.io.File(workDir));
        fc2.setDialogTitle("XML Datei zum Speichern auswhlen");
        fc2.addChoosableFileFilter(filter2);
        fc2.setFileFilter(filter2);
        auswahl2 = fc2.showOpenDialog(this);

        carbonresultXML = fc2.getSelectedFile().getPath();
*/
// CarbonCalc aufrufen
        CarbonCalc ccalc = new CarbonCalc();
        ccalc.calculate(st,ls, selectedFiles, biomassXML, cascadeXML, carbonresultXML);
        XmlToHtml xmltohtml = new XmlToHtml();
        String xslfile = progDir+System.getProperty("file.separator")+"moduls"+System.getProperty("file.separator")
                +"carbon"+System.getProperty("file.separator")+"carbonresult.xsl";
        String htmlfile = carbonresultXML.replace(".xml", ".html");
        xmltohtml.convert(carbonresultXML, xslfile, htmlfile);
// Anzeigen
        StartBrowser startBrowser = new StartBrowser(htmlfile);
        startBrowser.start();
        
        dispose();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        // TODO add your handling code here:
        String fileName = st.programDir + System.getProperty("file.separator") + "help" + System.getProperty("file.separator") + "FSUtilization.pdf";
        String seite = "file:" + System.getProperty("file.separator") + System.getProperty("file.separator") + System.getProperty("file.separator") + fileName;
        StartBrowser startBrowser = new StartBrowser(seite);
        startBrowser.start();

    }//GEN-LAST:event_jButton2ActionPerformed
    


    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel5;
    // End of variables declaration//GEN-END:variables
    
}
